@@ -83,26 +83,26 @@ void Parser::parseLine(ustring& data)
         */
 
         // Find prefix
-        ustring::size_type pos1 = data.find_first_of(" ", 1);
+        ustring::size_type pos1 = data.find(" ", 1);
         ustring from = data.substr(1, pos1 - 1);
 
         // Find command
-        ustring::size_type pos2 = data.find_first_of(" ", pos1 + 1);
+        ustring::size_type pos2 = data.find(" ", pos1 + 1);
         ustring command = data.substr(pos1 + 1, (pos2 - 1) - pos1);
 
         // Check whether there is any params
-        ustring::size_type pos3 = data.find_first_of(":", pos2 + 1);
+        ustring::size_type pos3 = data.find(" :", pos2);
         ustring param;
 
-        if ((pos3 - 1) != pos2) {
+        if (pos3 != pos2) {
             // We have params
-            param = data.substr(pos2 + 1, (pos3 - 2) - pos2);
+            param = data.substr(pos2 + 1, (pos3 - 1) - pos2);
         }
 
         // Get rest (whats after the ':', if there were any ':')
         ustring rest;
         if (pos3 != ustring::npos) {
-            rest = data.substr(pos3 + 1);
+            rest = data.substr(pos3 + 2);
         }
 
         #ifdef DEBUG
@@ -374,7 +374,7 @@ void Parser::Join(const ustring& nick, const ustring& param, const ustring& rest
 
     // Some clients/servers/bouncers might accidently send the channel name
     // in the 'rest' string, a bug there, but we would like to avoid a
-    // segfault here. I noticed the same hack in the xchat sources.
+    // segfault here. I noticed the same hack in the xchat source.
     if (chan.empty() && !rest.empty())
         chan = getWord(rest, 1);
 
@@ -511,12 +511,6 @@ void Parser::CMode(const ustring& from, const ustring& param)
     vector<ustring> arguments;
     Util::tokenizeWords(args, arguments);
 
-    if (arguments.empty()) {
-        // There were no further arguments, eg. MODE #channel +n
-        FE::emit(FE::get(CMODE) << findNick(from) << modes << channel, *chan, _conn);
-        return;
-    }
-
     std::vector<User> modesvec;
     bool sign = false; // Used to track whether we get a + or a -
 
@@ -545,23 +539,6 @@ void Parser::CMode(const ustring& from, const ustring& param)
                     FE::emit(FE::get(e) << findNick(from) << nick, *chan, _conn);
                 }
                 break;
-            case 'q':
-                {
-                    // This cause crashes, so its outcommented for now.
-                    /*
-                    Event e;
-                    IRC::UserMode mode = IRC::OWNER;
-                    sign ? (e = OWNERED) : (e = DEOWNERED);
-                    ustring nick = *arg_i++;
-
-                    User *user = chan->getUser(nick);
-                    sign ? (user->setMode(mode)) : (user->removeMode(mode));
-
-                    modesvec.push_back(*user);
-                    FE::emit(FE::get(e) << findNick (from) << nick, *chan, _conn);
-                    */
-                }
-                break;
             case 'o':
                 {
                 Event e;
@@ -627,14 +604,17 @@ void Parser::CMode(const ustring& from, const ustring& param)
 
                 modebuf[1] = (*i);
                 modebuf[2] = '\0';
+                if (isChannelMode(*i)) {
+                    FE::emit(FE::get(CMODE) << findNick(from) << modebuf << channel, *chan, _conn);
+                } else {
+                    ustring nick;
+                    if (arg_i == arguments.end())
+                          nick = *--arg_i;
+                    else
+                          nick = *arg_i++;
 
-                ustring nick;
-                if (arg_i == arguments.end())
-                      nick = *--arg_i;
-                else
-                      nick = *arg_i++;
-
-                FE::emit(FE::get(MODE) << findNick(from) << modebuf << nick, *chan, _conn);
+                    FE::emit(FE::get(MODE) << findNick(from) << modebuf << nick, *chan, _conn);
+                }
                 }
         }
 
@@ -724,11 +704,57 @@ void Parser::Banlist(const ustring& param)
 
     FE::emit(FE::get(BANLIST) << banmask << owner, *c, _conn);
 }
+
+// Function to parse the 005 numeric string, also known as RPL_ISUPPORT
+// Right now only a small subset of the parameters are being used, namely:
+//
+// CHANTYPES, PREFIX, CHANMODES, NETWORK
+//
+// The specification can be found (at the time of writing) at:
+// http://www.irc.org/tech_docs/005.html
+// http://www.irc.org/tech_docs/draft-brocklesby-irc-isupport-03.txt
+void Parser::parseIRCSupports(const ustring& supports)
+{
+    ustring::size_type pos;
+    pos = supports.find("CHANTYPES=");
+    if (pos != ustring::npos) {
+        pos += 10;
+        ustring::size_type endpos = supports.find_first_of(" ", pos);
+        _conn->supports.chantypes = supports.substr(pos, endpos - pos);
+        
+    }
+    pos = supports.find("PREFIX=");
+    if (pos != ustring::npos) {
+        pos += 7;
+        ustring::size_type endpos = supports.find_first_of(" ", pos);
+        _conn->supports.prefix = supports.substr(pos, endpos - pos);
+    }
+    pos = supports.find("CHANMODES=");
+    if (pos != ustring::npos) {
+        pos += 10;
+        ustring::size_type endpos = supports.find_first_of(" ", pos);
+        _conn->supports.chanmodes = supports.substr(pos, endpos - pos);
+    }
+    pos = supports.find("NETWORK=");
+    if (pos != ustring::npos) {
+        pos += 8;
+        ustring::size_type endpos = supports.find_first_of(" ", pos);
+        _conn->supports.network = supports.substr(pos, endpos - pos);
+    }
+}
      
 void Parser::numeric(int n, const ustring& from, const ustring& param, const ustring& rest)
 {
     switch(n)
     {
+        case 5:   // RPL_ISUPPORT
+            parseIRCSupports(skipFirstWord(param));
+        case 4:   // RPL_MYINFO
+        case 252: // RPL_LUSEROP
+        case 253: // RPL_LUSERUNKNOWN
+        case 254: // RPL_LUSERCHANNELS
+            FE::emit(FE::get(SERVERMSG1) << skipFirstWord(param) << rest, FE::CURRENT, _conn);
+            break;
         case 1:   // RPL_WELCOME
             _conn->Session.servername = from;
             _conn->Session.hasRegistered = true;
@@ -737,12 +763,7 @@ void Parser::numeric(int n, const ustring& from, const ustring& param, const ust
             App->fe->connected(_conn);
         case 2:   // RPL_YOURHOST
         case 3:   // RPL_CREATED
-        case 4:   // RPL_MYINFO
-        case 5:   // RPL_MYINFO
         case 251: // RPL_LUSERCLIENT
-        case 252: // RPL_LUSEROP
-        case 253: // RPL_LUSERUNKNOWN
-        case 254: // RPL_LUSERCHANNELS
         case 255: // RPL_LUSERME
             FE::emit(FE::get(SERVERMSG1) << rest, FE::CURRENT, _conn);
             break;
@@ -943,6 +964,13 @@ bool Parser::shouldHighlight(const ustring& str)
     return false;
 }
 
+bool Parser::isChannelMode(char m)
+{
+    if (_conn->supports.chanmodes.find_first_of(m) != ustring::npos)
+          return true;
+    return false;
+}
+
 ustring getWord(const ustring& str, int n)
 {
     int count = 0;
